"use strict";
/*
 * MIT License
 *
 * Copyright (c) 2017 Choko (choko@curioswitch.org)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const promisify = require("es6-promisify");
const google = require("googleapis");
const parseDuration = require("parse-duration");
const MILLIS_IN_SECOND = 1000;
class GoogleApis {
    getProjectId() {
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield this.init();
            return client.projectId;
        });
    }
    getProjectNumber() {
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield this.init();
            const { projectNumber } = yield client.getGoogleProject({
                projectId: client.projectId,
            });
            return projectNumber;
        });
    }
    createKeyring(location, keyring) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield this.init();
            const request = {
                parent: `projects/${client.projectId}/locations/${location}`,
                keyRingId: keyring,
            };
            const created = yield client.kms.createKeyring(request);
            console.log(`Keyring ${created.name} created.`);
        });
    }
    createKey(location, keyring, key) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield this.init();
            const request = {
                parent: `projects/${client.projectId}/locations/${location}/keyRings/${keyring}`,
                cryptoKeyId: key,
                resource: {
                    purpose: 'ENCRYPT_DECRYPT',
                },
            };
            const created = yield client.kms.createKey(request);
            console.log(`Key ${created.name} created.`);
        });
    }
    decryptKey(location, keyring, key, encryptedBase64) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield this.init();
            const request = {
                name: `projects/${client.projectId}/locations/${location}/keyRings/${keyring}/cryptoKeys/${key}`,
                resource: {
                    ciphertext: encryptedBase64,
                },
            };
            const { plaintext } = yield client.kms.decrypt(request);
            return plaintext;
        });
    }
    encryptKey(location, keyring, key, plaintext) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield this.init();
            const request = {
                name: `projects/${client.projectId}/locations/${location}/keyRings/${keyring}/cryptoKeys/${key}`,
                resource: {
                    plaintext: Buffer.from(plaintext).toString('base64'),
                },
            };
            const { ciphertext } = yield client.kms.encrypt(request);
            return ciphertext;
        });
    }
    setDecrypters(location, keyring, serviceAccounts) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield this.init();
            const request = {
                resource_: `projects/${client.projectId}/locations/${location}/keyRings/${keyring}`,
                resource: {
                    policy: {
                        bindings: [
                            {
                                members: serviceAccounts.map((acc) => `serviceAccount:${acc}`),
                                role: 'roles/cloudkms.cryptoKeyDecrypter',
                            },
                        ],
                    },
                },
            };
            yield client.kms.setIamPolicy(request);
        });
    }
    cancelCloudbuild(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield this.init();
            const request = {
                id,
                projectId: client.projectId,
            };
            yield client.cloudbuild.cancel(request);
        });
    }
    listCloudbuilds(filter) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield this.init();
            const request = {
                filter,
                projectId: client.projectId,
            };
            const { builds } = yield client.cloudbuild.list(request);
            return builds || [];
        });
    }
    startCloudbuild(cloudbuildConfig, substitutions, tags) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = yield this.init();
            const sanitizedConfig = cloudbuildConfig.timeout
                ? Object.assign({}, cloudbuildConfig, { timeout: `${parseDuration(cloudbuildConfig.timeout) /
                        MILLIS_IN_SECOND}s` }) : cloudbuildConfig;
            const request = {
                projectId: client.projectId,
                resource: Object.assign({}, sanitizedConfig, { options: Object.assign({}, sanitizedConfig.options, { substitutionOption: 'ALLOW_LOOSE' }), substitutions,
                    tags }),
            };
            console.log('Creating build job.');
            yield client.cloudbuild.create(request);
        });
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.client) {
                return this.client;
            }
            const { auth, projectId } = yield this.authorize();
            const cloudkms = google.cloudkms({ version: 'v1', auth });
            const cloudbuild = google.cloudbuild({ version: 'v1', auth });
            const resourceManager = google.cloudresourcemanager({
                auth,
                version: 'v1',
            });
            return {
                getGoogleProject: promisify(resourceManager.projects.get, resourceManager),
                cloudbuild: {
                    cancel: promisify(cloudbuild.projects.builds.cancel, cloudbuild),
                    create: promisify(cloudbuild.projects.builds.create, cloudbuild),
                    list: promisify(cloudbuild.projects.builds.list, cloudbuild),
                },
                kms: {
                    createKey: promisify(cloudkms.projects.locations.keyRings.cryptoKeys.create, cloudkms),
                    createKeyring: promisify(cloudkms.projects.locations.keyRings.create, cloudkms),
                    decrypt: promisify(cloudkms.projects.locations.keyRings.cryptoKeys.decrypt, cloudkms),
                    encrypt: promisify(cloudkms.projects.locations.keyRings.cryptoKeys.encrypt, cloudkms),
                    setIamPolicy: promisify(cloudkms.projects.locations.keyRings.cryptoKeys.setIamPolicy, cloudkms),
                },
                projectId,
            };
        });
    }
    authorize() {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => google.auth.getApplicationDefault((err, authClient, projectId) => {
                if (err) {
                    reject(err);
                    return;
                }
                resolve({
                    auth: authClient,
                    projectId,
                });
            }));
        });
    }
}
exports.GoogleApis = GoogleApis;
exports.googleApis = new GoogleApis();
//# sourceMappingURL=gcloud.js.map