"use strict";
/*
 * MIT License
 *
 * Copyright (c) 2017 Choko (choko@curioswitch.org)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const request = require("request-promise-native");
const keymanager_1 = require("./keymanager");
const constants_1 = require("./constants");
const GITHUB_API_BASE = 'https://api.github.com';
const GITHUB_URL_REPO_REGEX = /repos\/([^/]+\/[^/]+)\//;
function statusToState(status) {
    switch (status) {
        case 'QUEUED':
        case 'WORKING':
            return 'pending';
        case 'SUCCESS':
            return 'success';
        case 'FAILURE':
            return 'failure';
        default:
            return 'error';
    }
}
function statusToDescription(status) {
    switch (status) {
        case 'QUEUED':
            return 'Build queued.';
        case 'WORKING':
            return 'Build started.';
        case 'SUCCESS':
            return 'Build succeeded.';
        case 'FAILURE':
            return 'Build failed.';
        case 'INTERNAL_ERROR':
            return 'Build internal error.';
        case 'TIMEOUT':
            return 'Build timed out.';
        case 'CANCELLED':
            return 'Build cancelled.';
        default:
            return 'Build status unknown.';
    }
}
function makeRequest(uri, body) {
    return __awaiter(this, void 0, void 0, function* () {
        const repoMatch = GITHUB_URL_REPO_REGEX.exec(uri);
        if (!repoMatch) {
            throw new Error('Could not parse github url.');
        }
        const githubToken = yield keymanager_1.keyManager.getGithubToken(repoMatch[1]);
        return request({
            method: 'POST',
            uri,
            headers: {
                'User-Agent': 'cloudbuild-github',
            },
            body,
            json: true,
            auth: {
                pass: githubToken,
                user: 'token',
            },
        });
    });
}
function handleBuildEvent(event) {
    return __awaiter(this, void 0, void 0, function* () {
        const build = JSON.parse(Buffer.from(event.data.data, 'base64').toString('utf8'));
        let repoName = null;
        let revisionId = null;
        if (!build.substitutions) {
            // No substitutions for triggered builds, we'll post to the commit instead.
            const repoSource = build.sourceProvenance.resolvedRepoSource;
            // eslint-disable-next-line prefer-destructuring
            const cloudbuildRepoName = repoSource.repoName;
            if (!cloudbuildRepoName.startsWith('github-')) {
                // Not a github repo.
                return;
            }
            repoName = cloudbuildRepoName.substring('github-'.length).replace('-', '/');
            revisionId = repoSource.commitSha;
        }
        const statusesUrl = repoName && revisionId
            ? `${GITHUB_API_BASE}/repos/${repoName}/statuses/${revisionId}`
            : build.substitutions[constants_1.STATUSES_URL_KEY];
        if (!statusesUrl) {
            // A non-triggered build not from the webhook, nothing to do.
            return;
        }
        const status = {
            state: statusToState(build.status),
            target_url: build.logUrl,
            description: statusToDescription(build.status),
            context: 'ci/cloudbuild',
        };
        const statusResponse = yield makeRequest(statusesUrl, status);
        if (!statusResponse.state) {
            throw new Error(`Failed to set status: ${JSON.stringify(statusResponse)}`);
        }
        const commentsUrl = repoName && revisionId
            ? `${GITHUB_API_BASE}/repos/${repoName}/commits/${revisionId}/comments`
            : build.substitutions[constants_1.COMMENTS_URL_KEY];
        if (!commentsUrl) {
            // A non-triggered build not from the webhook, nothing to do (we shouldn't
            // actually get here since we have a similar check for statuses).
            return;
        }
        let comment;
        switch (build.status) {
            case 'QUEUED':
            case 'WORKING':
            case 'CANCELLED':
            case 'STATUS_UNKNOWN':
                return;
            case 'SUCCESS': {
                if (repoName) {
                    // Don't comment on success for triggered builds.
                    return;
                }
                comment = `Build succeded. If you have approval, you're ready to merge!\n\nLogs:\n${build.logUrl}`;
                break;
            }
            case 'FAILURE':
                comment = `Build failed. Check the logs and try again.\n\nLogs:\n${build.logUrl}`;
                break;
            default:
                comment = `Build terminated with unknown error. You may want to retry.\n\nLogs:\n${build.logUrl}`;
                break;
        }
        const commentResponse = yield makeRequest(commentsUrl, { body: comment });
        if (!commentResponse.id) {
            throw new Error(`Failed to set comment: ${JSON.stringify(commentResponse)}`);
        }
    });
}
exports.handleBuildEvent = handleBuildEvent;
//# sourceMappingURL=notifier.js.map